<?php
//
//  Nagios Core Config Manager
//  Copyright (c) 2010-2018 Nagios Enterprises, LLC
//
//  File: mysqldb.class.php
//  Desc: Includes all the features for integration with a MySQL Server.
//


class mysqldb {

    var $DB;
    var $arrSettings;
    var $strDBError       = "";
    var $error            = false;
    var $intLastId        = 0;
    var $intAffectedRows  = 0;

    /**
     * Constructor for the data class. You can pass a configuration
     * array or the config array will be pulled from the session.
     * (Can someone explain why $CFG is passed as a session value... and named SETS?)
     *
     * @param   array   $config     The configuration array (optional)
     * @param   int     $domain     For multiple domain configs (optional - not used)
     */
    function __construct($config=null, $domain=1)
    {
        // Set array settings
        $this->arrSettings = $config;
        if ($this->arrSettings === null) {
            $this->arrSettings = $_SESSION['SETS'];
        }

        // Connect to the database
        $this->getDatabase($this->arrSettings['db']['database']);
    }

    /**
     * Connect to MySQL and select the database we want
     *
     * @param   string  $database   Name of database to grab
     * @return  bool                True if success or false if failure
     */
    function getDatabase($database)
    {
        $server = $this->arrSettings['db']['server'];
        $port = $this->arrSettings['db']['port'];
        $username = $this->arrSettings['db']['username'];
        $password = $this->arrSettings['db']['password'];

        $this->dbconnect($server, $port, $username, $password, $database);

        if ($this->error == true) {
            return false;
        }
        return true;
    }

    /**
     * Sends an SQL statement to the database and stores the first returned
     *
     * @param   string  $strSQL     A SQL statement
     * @return  mixed               Value of field or false if failure
     */
    function getFieldData($strSQL)
    {
        $resQuery = $this->DB->query($strSQL);

        // Error handling 
        if ($resQuery && ($resQuery->num_rows != 0) && ($this->DB->error == "")) {
          $row = $resQuery->fetch_row();
          return $row[0];
        } else if ($this->DB->error != "") {
            $this->strDBError = $this->DB->error;
            $this->error = true;
            return false;
        }
        return "";
    }

    /**
     * Gets a single record and returns it as an associate array
     *
     * @param   string  $strSQL         An SQL statement
     * @param   array   $arrDataset     Associative array of dataset selected by SQL query (reference)
     * @return  bool                    True if success or false if failure
     */
    function getSingleDataset($strSQL, &$arrDataset)
    {
        $arrDataset = array();
        $resQuery = $this->DB->query($strSQL);
    
        // Error handling
        if ($resQuery && ($resQuery->num_rows != 0) && ($this->DB->error == "")) {
            $arrDataset = $resQuery->fetch_assoc();
            return true;
        } else if ($this->DB->error != "") {
            $this->strDBError = $this->DB->error;
            $this->error = true;
            return false;
        }
        return true;
    }

    /**
     * Get an array of records and store them numerically
     *
     * @param   string  $strSQL         An SQL statement
     * @param   array   $arrDataset     Associative array of dataset selected by SQL query (reference)
     * @param   int     $intDataCount   The number of records to get (reference)
     *
     * @return bool
     */
    function getDataArray($strSQL, &$arrDataset, &$intDataCount)
    {
        $arrDataset = array();
        $intDataCount = 0;
        $resQuery = $this->DB->query($strSQL);
    
        // Get the data and return it or handle the errors
        if ($resQuery && ($resQuery->num_rows != 0) && ($this->DB->error == "")) {
            $intDataCount = $resQuery->num_rows;
            $i = 0;
            while ($arrDataTemp = $resQuery->fetch_assoc()) {
                foreach ($arrDataTemp AS $key => $value) {
                    $arrDataset[$i][$key] = $value;
                }
                $i++;
            }
            return true;
        } else if ($this->DB->error != "") {
            $this->strDBError = $this->DB->error;
            $this->error = true;
            return false;
        }
        return true;
    }

    /**
     * Inserts data into the database
     * Also sets:
     *      $this->intLastId - ID of the generated data set
     *      $this->intAffectedRows - Number of records affected
     *
     * @param   string  $strSQL     An SQL statement
     * @return  bool                True on success or false on failure
     */
    function insertData($strSQL)
    {
        $resQuery = $this->DB->query($strSQL);
        
        // Error handling
        if ($this->DB->error == "") {
              $this->intLastId = $this->DB->insert_id;
              $this->intAffectedRows = $this->DB->affected_rows;
              return true;
        } else {
            $this->strDBError = $this->DB->error;
            $this->error = true;
            return false;
        }
    }

    /**
     * Count the number of data rows from a query
     * (On fialure it also sets "error" value in the object)
     *
     * @param   string  $strSQL     An SQL statement
     * @return  int                 Number of rows on success or 0 on failure
     */
    function countRows($strSQL)
    {
        $resQuery = $this->db->query($strSQL);

        // Error handling
        if ($resQuery && ($this->DB->error == "")) {
            return $resQuery->num_rows;
        } else {
            $this->strDBError = $this->DB->error;
            $this->error = true;
            return 0;
        }
    }

    /**
     * Connect to the database server restore
     *
     * @param $dbserver
     * @param $dbport
     * @param $dbuser
     * @param $dbpasswd
     *
     * @return  bool        True on success or false on failure
     */
    function dbconnect($dbserver, $dbport, $dbuser, $dbpasswd, $database)
    {
        // Parameters are missing
        if (($dbserver == "") || ($dbuser == "")) {
            $this->strDBError = _("Missing server connection parameter!")."<br>\n";
            $this->error = true;
            return false;
        }
        
        // Connect to DB
        $this->DB = new MySQLi($dbserver, $dbuser, $dbpasswd, $database, $dbport);
        
        // Connection failed so give us an error
        if ($this->DB->connect_error) {
            $this->strDBError = "[".$this->arrSettings['db']['server']."] "._("Connection to the database server has failed by reason:")."<br>\n";
            $this->strDBError .= $this->DB->connect_errorno.": ".$this->DB->connect_error."\n";
            $this->error = true;
            return false;
        }

        // Set charset
        $this->DB->set_charset("utf8");
        $this->DB->query("set names 'utf8'");

        return true;
    }

    /**
     * Close the connection to the database server
     *
     * @return  bool        True on success or false on failure
     */
    function dbdisconnect()
    {
        $this->DB->close();
        return true;
    }
}
