<?php

define('RECURRINGDOWNTIME_CFG', '/usr/local/nagios/etc/recurringdowntime.cfg');

/**
 * Convert the string form of the recurring downtime config to a PHP array
 * (Possibly convert this into a regex? Pull out whatever is between {x} instead)
 *
 * @param $str_cfg
 * @return array
 */
function recurringdowntime_cfg_to_array($str_cfg)
{
    $schedules = preg_split('/define schedule \{([\s]+)/', $str_cfg);
    $ret = array();
    if (count($schedules) > 0) {
        foreach ($schedules as $k => $schedule) {
            $lines = explode("\n", $schedule);
            $schedule_arr = array();
            foreach ($lines as $k2 => $line) {
                if ($line == "" || $line == "}") {
                    continue;
                }
                $line_arr = array();
                foreach (explode("\t", trim($line)) as $k3 => $val) {
                    if ($val != "") {
                        $line_arr[] = trim($val);
                    }
                }
                if ($line_arr[0] == "sid") {
                    $sid = isset($line_arr[1]) ? $line_arr[1] : "";
                } else {
                    $val = isset($line_arr[1]) ? $line_arr[1] : "";
                    $schedule_arr[$line_arr[0]] = $val;
                }
            }
            if (count($schedule_arr) > 0) {
                $ret[$sid] = $schedule_arr;
            }
        }
    }
    return $ret;
}


/**
 * Convert a PHP array containing recurring downtime config to a string appropriate for
 * downtime's schedule.cfg
 *
 * @param $arr
 * @return string
 */
function recurringdowntime_array_to_cfg($arr)
{
    if (count($arr) == 0) {
        return "";
    }
    $cfg_str = "";
    foreach ($arr as $sid => $schedule) {
        if (count($schedule) == 0) {
            continue;
        }
        $cfg_str .= "define schedule {\n";
        $cfg_str .= "\tsid\t\t\t$sid\n";
        foreach ($schedule as $var => $val) {

            // get a sane tab count for proper viewing/troubleshooting
            $tabs = "\t\t";
            if ($var == 'servicegroup_name' || $var == 'service_description')
                $tabs = "\t";
            if ($var == 'user' || $var == 'comment' || $var == 'time' || $var == 'svcalso')
                $tabs .= "\t";

            $cfg_str .= "\t$var{$tabs}$val\n";
        }
        $cfg_str .= "}\n\n";
    }
    return $cfg_str;
}


/**
 * Write the configuration to disk
 *
 * @param $cfg
 * @return bool
 */
function recurringdowntime_write_cfg($cfg)
{
    if (is_array($cfg)) {
        $cfg_str = recurringdowntime_array_to_cfg($cfg);
    } else {
        $cfg_str = $cfg;
    }
    recurringdowntime_check_cfg();
    file_put_contents(RECURRINGDOWNTIME_CFG, $cfg_str);
    return true;
}


/*
 * Make sure cfg file exists, and if not, create it.
 */
function recurringdowntime_check_cfg()
{
    if (!file_exists(RECURRINGDOWNTIME_CFG)) {
        $fh = @fopen(RECURRINGDOWNTIME_CFG, "w+");
        fclose($fh);
    }
}


/**
 * Get config from cfg file
 *
 * @return array
 */
function recurringdowntime_get_cfg()
{
    recurringdowntime_check_cfg();
    $cfg = file_get_contents(RECURRINGDOWNTIME_CFG);
    return recurringdowntime_cfg_to_array($cfg);
}


/**
 * Get downtime schedule for specified host
 *
 * @param bool $host
 * @return array
 */
function recurringdowntime_get_host_cfg($host = false)
{
    $cfg = recurringdowntime_get_cfg();
    $ret = array();
    foreach ($cfg as $sid => $schedule) {
        if (array_key_exists('schedule_type', $schedule)) {
            if ($schedule["schedule_type"] == "hostgroup") {
                continue;
            }
            if ($schedule["schedule_type"] == "servicegroup") {
                continue;
            }
            if ($schedule["schedule_type"] == "service") {
                continue;
            }
        }
        if ($host && !(strtolower($schedule["host_name"]) == strtolower($host))) {
            continue;
        }
        if (array_key_exists('host_name', $schedule)) {
            if (is_authorized_for_host(0, $schedule["host_name"])) {
                $ret[$sid] = $schedule;
            }
        }
    }
    return $ret;
}


/**
 * Get downtime schedule for specified host
 *
 * @param bool $host
 * @return array
 */
function recurringdowntime_get_service_cfg($host = false)
{
    $cfg = recurringdowntime_get_cfg();
    $ret = array();
    foreach ($cfg as $sid => $schedule) {
        if (array_key_exists('schedule_type', $schedule)) {
            if ($schedule["schedule_type"] == "hostgroup") {
                continue;
            }
            if ($schedule["schedule_type"] == "servicegroup") {
                continue;
            }
            if ($schedule["schedule_type"] == "host") {
                continue;
            }
        }

        if ($host && !(strtolower($schedule["host_name"]) == strtolower($host))) {
            continue;
        }

        // Commented out because $request isn't even defined which means this never does anything
        //if (strstr($request["service_description"], "*")) {
        //    $search_str = "lk:" . str_replace("*", "%", $request["service_description"]);
        //} else {
        //    $search_str = $request["service_description"];
        //}

        if (array_key_exists('host_name', $schedule)) {
            if (!is_authorized_for_service(0, $schedule["host_name"], $search_str)) {
                continue;
            }

            if (is_authorized_for_host(0, $schedule["host_name"])) {
                $ret[$sid] = $schedule;
            }
        }
    }
    return $ret;
}


/**
 * Get downtime schedule for specified hostgroup
 *
 * @param bool $hostgroup
 * @return array
 */
function recurringdowntime_get_hostgroup_cfg($hostgroup = false)
{
    $cfg = recurringdowntime_get_cfg();
    $ret = array();
    foreach ($cfg as $sid => $schedule) {
        if (array_key_exists('schedule_type', $schedule)) {
            if ($schedule["schedule_type"] != "hostgroup") {
                continue;
            }
        }
        if ($hostgroup && !(strtolower($schedule["hostgroup_name"]) == strtolower($hostgroup))) {
            continue;
        }
        if (array_key_exists('hostgroup_name', $schedule)) {
            if (is_authorized_for_hostgroup(0, $schedule["hostgroup_name"])) {
                $ret[$sid] = $schedule;
            }
        }
    }
    return $ret;
}


/**
 * Get downtime schedule for specified servicegroup
 *
 * @param bool $servicegroup
 * @return array
 */
function recurringdowntime_get_servicegroup_cfg($servicegroup = false)
{
    $cfg = recurringdowntime_get_cfg();
    $ret = array();
    foreach ($cfg as $sid => $schedule) {
        if (array_key_exists('schedule_type', $schedule)) {
            if ($schedule["schedule_type"] != "servicegroup") {
                continue;
            }
        }
        if ($servicegroup && !(strtolower($schedule["servicegroup_name"]) == strtolower($servicegroup))) {
            continue;
        }
        if (array_key_exists('servicegroup_name', $schedule)) {
            if (is_authorized_for_servicegroup(0, $schedule["servicegroup_name"])) {
                $ret[$sid] = $schedule;
            }
        }
    }
    return $ret;
}
